classdef soundSweepAppc < matlab.apps.AppBase

    %
    %
    %          Sound Sweep         
    %   Copyright 2020 Thomas P Weldon
    %
    %  UI allowing user to change frequency and amplitude
    %
    % Properties that correspond to app components
    properties (Access = public)
        SweepSoundApp            matlab.ui.Figure
        GridLayout               matlab.ui.container.GridLayout
        LeftPanel                matlab.ui.container.Panel
        FrequencyLabel           matlab.ui.control.Label
        freqEditField            matlab.ui.control.NumericEditField
        AmplitudedBLabel         matlab.ui.control.Label
        amplitudeEditField       matlab.ui.control.NumericEditField
        FreqkHzSliderLabel       matlab.ui.control.Label
        FreqkHzSlider            matlab.ui.control.Slider
        AmplSliderLabel          matlab.ui.control.Label
        AmplSlider               matlab.ui.control.Slider
        RightPanel               matlab.ui.container.Panel
        MyAxesUIAxes  matlab.ui.control.UIAxes
    sampleRate=44100
    samplesPerFrame=1024*2
    sineSource 
    deviceWriter
    deviceReader
    frequency=0
    amplitude=10^(-20/20)
    end

    % Properties that correspond to apps with auto-reflow
    properties (Access = private)
        onePanelWidth = 576;
    end

    % Callbacks that handle component events
    methods (Access = private)

        % Changes arrangement of the app based on UIFigure width
        function updateAppLayout(app, event)
            currentFigureWidth = app.SweepSoundApp.Position(3);
            if(currentFigureWidth <= app.onePanelWidth)
                % Change to a 2x1 grid
                app.GridLayout.RowHeight = {316, 316};
                app.GridLayout.ColumnWidth = {'1x'};
                app.RightPanel.Layout.Row = 2;
                app.RightPanel.Layout.Column = 1;
            else
                % Change to a 1x2 grid
                app.GridLayout.RowHeight = {'1x'};
                app.GridLayout.ColumnWidth = {257, '1x'};
                app.RightPanel.Layout.Row = 1;
                app.RightPanel.Layout.Column = 2;
            end
        end

        function createSineSource(app)
            app.sineSource  = audioOscillator( ...
                  'Frequency', app.freqEditField.Value, ...
                   'Amplitude', 10^(app.amplitudeEditField.Value/20), ...
                    'SignalType', 'sine', ...
                     'SampleRate', app.sampleRate, ...
                    'SamplesPerFrame', app.samplesPerFrame);        
        end
        
        function createDeviceWriter(app)
            app.deviceWriter = audioDeviceWriter;
            getAudioDevices(app.deviceWriter)
            %app.deviceWriter.Device='S111 BT';   % bluetooth speaker purchased for this course
            app.deviceWriter.Device='Default';
            %deviceWriter.Device='Built-in Output';
            app.deviceWriter.SampleRate=app.sampleRate;
            app.deviceWriter.ChannelMappingSource='Property';
            app.deviceWriter.ChannelMapping=1;

        end
 
           
%microphone input

       function  createDeviceReader(app)
            app.deviceReader = audioDeviceReader;
            getAudioDevices(app.deviceReader)
%             app.deviceReader.Device='Built-in Microphone'; %
             app.deviceReader.Device='Default'; %
           app.deviceReader.SampleRate=app.sampleRate;
            app.deviceReader.SamplesPerFrame=app.samplesPerFrame;
       end
 


        % Value changed function: freqEditField
        function FreqBoxChanged(app, event)
            value = app.freqEditField.Value;
            app.frequency=value;
            app.FreqkHzSlider.Value=(app.frequency/20000)^(1/3)*20;
            
            app.sineSource.Frequency=app.frequency;
            tic
            while toc<0.2
                 x = app.sineSource(); app.deviceWriter(x);
            end
            
            xx=[app.frequency/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;
            
         
        end
        
  

        % Value changed function: FreqkHzSlider
        function FreqSliderInput(app, event)
            value = app.FreqkHzSlider.Value;
            app.frequency=(value/20)^3*20000;
            app.freqEditField.Value=round(app.frequency);
            %fplot(app, event);

            app.sineSource.Frequency=app.frequency;
            tic
            while toc<0.2
                 x = app.sineSource(); app.deviceWriter(x);
            end

            xx=[app.frequency/1000 app.frequency/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;
     
       end

        % Value changing function: FreqkHzSlider
        function FreqSliderChanging(app, event)
            changingValue = event.Value;
            app.frequency=(event.Value/20)^3*20000;
            app.freqEditField.Value=round(app.frequency);
            app.sineSource.Frequency=app.frequency;
            x = app.sineSource(); app.deviceWriter(x);    
        end

        % Value changed function: amplitudeEditField
        function AmplitudeBoxChanged(app, event)
            value = app.amplitudeEditField.Value;
            app.AmplSlider.Value=value;
            app.amplitude=10^(app.amplitudeEditField.Value/20);
            app.sineSource.Amplitude=app.amplitude;
            tic
            while toc<0.2
                 x = app.sineSource(); app.deviceWriter(x);
            end

             xx=[app.freqEditField.Value/1000 app.freqEditField.Value/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;
           
        end

        % Value changed function: AmplSlider
        function AmplSliderChanged(app, event)
            value = app.AmplSlider.Value
            app.amplitudeEditField.Value=value;
            app.amplitude=10^(app.amplitudeEditField.Value/20);
            app.sineSource.Amplitude=app.amplitude;
            tic
            while toc<0.2
                 x = app.sineSource(); app.deviceWriter(x);
            end

            xx=[app.freqEditField.Value/1000 app.freqEditField.Value/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;

            
        end

        % Value changing function: AmplSlider
        function AmplSliderChanging(app, event)
            %fplot(app, event);
           changingValue = event.Value
            app.amplitudeEditField.Value=event.Value;
            app.amplitude=10^(app.amplitudeEditField.Value/20);
            app.sineSource.Amplitude=app.amplitude;
            x = app.sineSource(); app.deviceWriter(x);

            
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create SweepSoundApp and hide until all components are created
            app.SweepSoundApp = uifigure('Visible', 'off');
            app.SweepSoundApp.AutoResizeChildren = 'off';
            app.SweepSoundApp.Position = [100 100 653 316];
            app.SweepSoundApp.Name = 'Sound Sweep         Copyright 2020 Thomas P Weldon';
            app.SweepSoundApp.SizeChangedFcn = createCallbackFcn(app, @updateAppLayout, true);

            % Create GridLayout
            app.GridLayout = uigridlayout(app.SweepSoundApp);
            app.GridLayout.ColumnWidth = {257, '1x'};
            app.GridLayout.RowHeight = {'1x'};
            app.GridLayout.ColumnSpacing = 0;
            app.GridLayout.RowSpacing = 0;
            app.GridLayout.Padding = [0 0 0 0];
            app.GridLayout.Scrollable = 'on';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.GridLayout);
            app.LeftPanel.Layout.Row = 1;
            app.LeftPanel.Layout.Column = 1;
            app.LeftPanel.Scrollable = 'on';

            % Create FrequencyLabel
            app.FrequencyLabel = uilabel(app.LeftPanel);
            app.FrequencyLabel.HorizontalAlignment = 'right';
            app.FrequencyLabel.Position = [50 230 77 22];
            app.FrequencyLabel.Text = 'Frequency';

            % Create freqEditField
            app.freqEditField = uieditfield(app.LeftPanel, 'numeric');
            app.freqEditField.Limits = [0 20000];
            app.freqEditField.ValueDisplayFormat = '%8.f';
            app.freqEditField.ValueChangedFcn = createCallbackFcn(app, @FreqBoxChanged, true);
            app.freqEditField.Position = [142 230 100 22];
            app.freqEditField.Value = 150;

            % Create AmplitudedBLabel
            app.AmplitudedBLabel = uilabel(app.LeftPanel);
            app.AmplitudedBLabel.HorizontalAlignment = 'right';
            app.AmplitudedBLabel.Position = [39 113 88 22];
            app.AmplitudedBLabel.Text = 'Amplitude dB';

            % Create amplitudeEditField
            app.amplitudeEditField = uieditfield(app.LeftPanel, 'numeric');
            app.amplitudeEditField.Limits = [-100 0];
            app.amplitudeEditField.ValueChangedFcn = createCallbackFcn(app, @AmplitudeBoxChanged, true);
            app.amplitudeEditField.Position = [142 113 100 22];
            app.amplitudeEditField.Value = -20;

            % Create FreqkHzSliderLabel
            app.FreqkHzSliderLabel = uilabel(app.LeftPanel);
            app.FreqkHzSliderLabel.HorizontalAlignment = 'right';
            app.FreqkHzSliderLabel.Position = [2 182 54 22];
            app.FreqkHzSliderLabel.Text = 'Freq kHz';

            % Create FreqkHzSlider
            app.FreqkHzSlider = uislider(app.LeftPanel);
            app.FreqkHzSlider.Limits = [0 20];
            app.FreqkHzSlider.MajorTicks =[0 5 10 15 20];
            app.FreqkHzSlider.MajorTickLabels ={'0','0.3','2.5','9','20'};
            app.FreqkHzSlider.ValueChangedFcn = createCallbackFcn(app, @FreqSliderInput, true);
            app.FreqkHzSlider.ValueChangingFcn = createCallbackFcn(app, @FreqSliderChanging, true);
            app.FreqkHzSlider.Position = [77 191 150 3];
            app.FreqkHzSlider.Value = 0.150;

            % Create AmplSliderLabel
            app.AmplSliderLabel = uilabel(app.LeftPanel);
            app.AmplSliderLabel.HorizontalAlignment = 'right';
            app.AmplSliderLabel.Position = [24 69 37 22];
            app.AmplSliderLabel.Text = 'Ampl.';

            % Create AmplSlider
            app.AmplSlider = uislider(app.LeftPanel);
            app.AmplSlider.Limits = [-100 0];
            app.AmplSlider.ValueChangedFcn = createCallbackFcn(app, @AmplSliderChanged, true);
            app.AmplSlider.ValueChangingFcn = createCallbackFcn(app, @AmplSliderChanging, true);
            app.AmplSlider.Position = [82 78 150 3];

            % Create RightPanel
            app.RightPanel = uipanel(app.GridLayout);
            app.RightPanel.Layout.Row = 1;
            app.RightPanel.Layout.Column = 2;
            app.RightPanel.Scrollable = 'on';

            % Create MyAxesUIAxes
            app.MyAxesUIAxes = uiaxes(app.RightPanel);
            title(app.MyAxesUIAxes, 'Sound Level dB')
            xlabel(app.MyAxesUIAxes, 'Frequency')
            ylabel(app.MyAxesUIAxes, 'Amplitude dB')
            app.MyAxesUIAxes.XLim = [0 20];
            app.MyAxesUIAxes.YLim = [-100 0];
            app.MyAxesUIAxes.XTick = [0 4 8 12 16 20];
            app.MyAxesUIAxes.XTickLabel = {'0'; '4'; '8'; '12'; '16'; '20'};
            app.MyAxesUIAxes.XGrid = 'on';
            app.MyAxesUIAxes.YGrid = 'on';
            app.MyAxesUIAxes.Position = [30 36 326 250];

            % Show the figure after all components are created
            app.SweepSoundApp.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = soundSweepAppc

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.SweepSoundApp)
            
            xx=[app.freqEditField.Value/1000 app.freqEditField.Value/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;

createSineSource(app)
createDeviceReader(app)
createDeviceWriter(app)
 
  
  
            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)
            
        release(app.deviceReader)
        release(app.deviceWriter)


            % Delete UIFigure when app is deleted
            delete(app.SweepSoundApp)
        end
    end
end